package routines;

import javax.naming.*; 
import javax.naming.directory.*; 
import java.util.*; 
import java.security.*;

/*
 * user specification: the function's comment should contain keys as follows: 1. write about the function's comment.but
 * it must be before the "{talendTypes}" key.
 * 
 * 2. {talendTypes} 's value must be talend Type, it is required . its value should be one of: String, char | Character,
 * long | Long, int | Integer, boolean | Boolean, byte | Byte, Date, double | Double, float | Float, Object, short |
 * Short
 * 
 * 3. {Category} define a category for the Function. it is required. its value is user-defined .
 * 
 * 4. {param} 's format is: {param} <type>[(<default value or closed list values>)] <name>[ : <comment>]
 * 
 * <type> 's value should be one of: string, int, list, double, object, boolean, long, char, date. <name>'s value is the
 * Function's parameter name. the {param} is optional. so if you the Function without the parameters. the {param} don't
 * added. you can have many parameters for the Function.
 * 
 * 5. {example} gives a example for the Function. it is optional.
 */
public class ADConnection {
	DirContext ldapContext; 
	String ldapSearchBase;

	public ADConnection(String serverIP, String admin, String password, String base) { 
		Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider()); 
		// the keystore that holds trusted root certificates 
		Security.setProperty("ssl.SocketFactory.provider",LazySSLSocketFactory.class.getName());

		// System.setProperty("javax.net.ssl.keyStore", "D:\\Talend\\keystore"); 
		// System.setProperty("javax.net.debug", "all"); 
		try { 
			this.ldapSearchBase = base;
			Hashtable<Object,Object> ldapEnv = new Hashtable<Object,Object>(11); 
			ldapEnv.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory"); 
			ldapEnv.put(Context.PROVIDER_URL, "ldap://" + serverIP + ":636"); 
			ldapEnv.put(Context.SECURITY_AUTHENTICATION, "simple"); 
			ldapEnv.put(Context.SECURITY_PRINCIPAL, admin); 
			ldapEnv.put(Context.SECURITY_CREDENTIALS, password); 
			ldapEnv.put(Context.SECURITY_PROTOCOL, "ssl"); 
			ldapEnv.put( "java.naming.ldap.factory.socket", LazySSLSocketFactory.class.getName());
			
			ldapContext = new InitialDirContext(ldapEnv); 
		} 
		catch (Exception e) { 
			System.out.println(" bind error: " + e); 
			e.printStackTrace(); 
			System.exit(-1); 
		} 
	} 
	public void updatePassword(String cptLogin, String password) { 
		if (getDn(cptLogin) == null) return;
		try { 
			System.out.println("Password for " + getDn(cptLogin));
			String quotedPassword = "\"" + password + "\""; 
			char unicodePwd[] = quotedPassword.toCharArray(); 
			byte pwdArray[] = new byte[unicodePwd.length * 2]; 
			for (int i=0; i<unicodePwd.length; i++) { 
				pwdArray[i*2 + 1] = (byte) (unicodePwd[i] >>> 8); 
				pwdArray[i*2 + 0] = (byte) (unicodePwd[i] & 0xff); 
			} 
			ModificationItem[] mods = new ModificationItem[1]; 
			mods[0] = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
					new BasicAttribute("UnicodePwd", pwdArray)); 
			ldapContext.modifyAttributes(getDn(cptLogin), mods); 
		} 
		catch (Exception e) { 
			System.out.println("update password error: " + e); 
			System.exit(-1); 
		} 
	} 

	public String getDn(String cptLogin) {
		try { 
			String searchFilter = "(&(objectClass=user)(sAMAccountName=" + cptLogin + "))";

			SearchControls searchControls = new SearchControls();
			searchControls.setSearchScope(SearchControls.SUBTREE_SCOPE);

			NamingEnumeration<SearchResult> results = ldapContext.search(ldapSearchBase, searchFilter, searchControls);

			if(results.hasMoreElements()) {
				SearchResult searchResult = (SearchResult) results.nextElement();

				//make sure there is not another item available, there should be only 1 match
				if(results.hasMoreElements()) {
					System.err.println("Matched multiple groups for the group with SID: " + cptLogin);
					return null;
				} else {
					return (String)searchResult.getAttributes().get("distinguishedName").get();
				}
			}
			return null;

		} catch (Exception e) { 
				System.out.println("find DN error: " + e); 
				return null;
			} 
		}

	}
